import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.io.*;

public class BroadcastApplet extends Applet implements ActionListener {
	int port;
	String host;
	TextField inputField;
	TextArea outputArea;
	Socket socket;
	BufferedReader in;
	PrintWriter out;
	Thread readingThread;

	public void init() {
		inputField = new TextField();
		outputArea = new TextArea();
		setLayout(new BorderLayout());
		add(inputField, "North");
		add(outputArea, "Center");
		String param = getParameter("PORT");
		if (param == null)
			port = 8001;
		else
			port = Integer.parseInt(param);
		host = getParameter("HOST");
		if (host == null)
			host = getCodeBase().getHost();
		inputField.addActionListener(this);
		doLayout();
	}

	public void start() {
		if (socket == null)
			connect();
		readingThread = new Thread() {
			public void run() {
				if (socket == null)
					return;
				while (socket != null && readingThread == Thread.currentThread()) {
					try {
						String line = in.readLine();
						appendLine(line);
					} catch (IOException ie) {
						ie.printStackTrace();
						closeSocket();
					}
				}
			}
		};
		readingThread.start();
	}

	public void stop() {
		readingThread = null;
		closeSocket();
	}

	public void actionPerformed(ActionEvent ae) {
		if (ae.getSource() instanceof TextField) {
			String line = inputField.getText();
			inputField.setText("");
			if (socket == null)
				return;
			try {
				out.println(line);
				if (out.checkError())
					throw new IOException("Print error");
			} catch (IOException ie) { 
				closeSocket();
			}
		}
	}

	public void connect() {
		try {
			socket = new Socket(host, port);
			in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			out = new PrintWriter(new OutputStreamWriter(socket.getOutputStream()), true);
			appendLine("Connected...");
		} catch (UnknownHostException uhe) {
			appendLine("Unknown host");
		} catch (IOException ie) {
			ie.printStackTrace();
		}
	}

	public synchronized void closeSocket() {
		try {
			if (in != null)
				in.close();
		} catch (Exception e) {}
		if (out != null)
			out.close();
		try {
			if (socket != null)
				socket.close();
		} catch (Exception e) {
			socket = null;
		}
	}

	public synchronized void appendLine(String message) {
		outputArea.append(message+"\n");
	}
}
