import java.io.*;
import java.net.*;
import java.util.*;

class BroadcastServer {
	final static int PORT = 8001;
	ServerSocket server = null;
	Vector clients = new Vector();

	class Client extends Thread {
		Socket socket = null;
		BufferedReader in = null;
		PrintWriter out = null;

		Client (Socket socket) throws IOException {
			this.socket = socket;
			this.in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			this.out = new PrintWriter(new OutputStreamWriter(socket.getOutputStream()), true);
			start();
		}

		public void run() {
			try {
				while (true) {
					String line = in.readLine();
					if (line == null) {
						close();
						break;
					}

					System.out.println(getName()+':'+line);
					BroadcastServer.this.broadcast(line);
				}
			} catch (IOException ie) {
				ie.printStackTrace();
				close();
			}
		}

		public void close() {
			synchronized (BroadcastServer.this) {
				try {
					if (in != null)
						in.close();
				} catch (Exception e) {}
				if (out != null)
					out.close();
				try {
					if (socket != null)
						socket.close();
				} catch (Exception e) {}
				socket = null;
			}
		}
	}

	public BroadcastServer() {
		try {
			server = new ServerSocket(PORT);
		} catch (IOException ie) {
			ie.printStackTrace();
			System.exit(1);
		}
		while (true) {
			try {
				Socket socket = server.accept();
				Client client = new Client(socket);
				synchronized (this) {
					clients.addElement(client);
				}
			} catch (IOException ie) {
				ie.printStackTrace();
			}
		}
	}


	public static void main(String[] args) {
		new BroadcastServer();
	}

	public synchronized void broadcast(String message) {
		for (Enumeration enuma = clients.elements(); enuma.hasMoreElements(); ) {
			Client client = (Client) enuma.nextElement();
			if (client == null)
				continue;
			if (client.socket == null) {
				clients.removeElement(client);
				continue;
			}
			client.out.println(message);
			if (client.out.checkError()) {
				System.err.println("Error while sending message.");
				client.close();
			}
		}
	}
}
